## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)
opt <- options()
options(continue="  ", width=70, prompt=" ")
on.exit(options(opt))
library(DOVE3, quietly=TRUE)

## ----eval=FALSE---------------------------------------------------------------
#  outcome(subject_id, entry_time, event_time, censor_time)

## ----eval=FALSE---------------------------------------------------------------
#  exposure(vaccine_time, vaccine_type, infection_time, infection_type)

## ----eval=FALSE---------------------------------------------------------------
#  dove3(formula,
#        data,
#        vaccine_infection_interaction = FALSE,
#        vaccine_knots = NULL,
#        vaccine_uninfected_knots = NULL,
#        vaccine_infected_knots = NULL,
#        prior_infection_knots = NULL,
#        related_vaccine_types = NULL,
#        last_piece_constant = FALSE,
#        reinfection_cutoff = 14,
#        plots = TRUE
#  )

## ----intCens-usage, eval=FALSE------------------------------------------------
#  outcome(subject_id, entry_time, event_time, censor_time) ~ covariates +
#      exposure(vaccine_time, vaccine_type, infection_time, infection_type)

## ----data-load----------------------------------------------------------------
data(exampleData)

## ----data-head----------------------------------------------------------------
head(exampleData)

## ----data-summary-------------------------------------------------------------
summary(exampleData)

## ----eval=FALSE, echo=TRUE, message=FALSE, warning=FALSE----------------------
#  vaccine.knots = list("vac.type1" = c(30),
#                       "vac.type2" = c(30,60),
#                       "vac.type3" = c(30,60))
#  prior.infection.knots = list("inf.type1" = c(14),
#                               "inf.type2" = c(14))

## ----eval=FALSE, echo=TRUE, message=FALSE, warning=FALSE----------------------
#  formula = outcome(subject.id, entry.time, event.time, censor.time) ~
#    age + gender + priority +
#    exposure(Vtime, Vtype, infection.time, infection.type)
#  data = exampleData
#  
#  result1 = dove3(formula = formula,
#                  data = data,
#                  vaccine_knots = vaccine.knots,
#                  prior_infection_knots = prior.infection.knots)

## ----echo=FALSE---------------------------------------------------------------
load(file = "results1.RData")

## -----------------------------------------------------------------------------
result1$covariates

## -----------------------------------------------------------------------------
head(result1$effectiveness[[1]])

## ----eval=FALSE---------------------------------------------------------------
#  result1$plots[[1]]

## ----dove3-no-interaction-plot, echo=FALSE, out.width='70%',fig.cap='\\label{fig:dove3Figs1}Plots auto-generated by \\textit{dove3()}. The estimated effectiveness curve (black line) without interaction assumption and its $95\\%$ confidence intervals (grey shadow) are shown as a function of the time since vaccination.' , fig.show='hold', fig.align='center'----
knitr::include_graphics(
  path=c("plot1.pdf"),
  auto_pdf = getOption("knitr.graphics.auto_pdf", FALSE),
  dpi = NULL,
  error = getOption("knitr.graphics.error", TRUE)
)

## ----eval=FALSE, echo=TRUE, message=FALSE, warning=FALSE----------------------
#  vaccine.uninfected.knots = list("vac.noinf.type1" = c(30),
#                                  "vac.noinf.type2" = c(30,60),
#                                  "vac.noinf.type3" = c(30,60))
#  
#  vaccine.infected.knots = list("vac.noinf.type1" = c(30),
#                                "vac.noinf.type2" = c(60),
#                                "vac.noinf.type3" = c(60))

## ----eval=FALSE, echo=TRUE, message=FALSE, warning=FALSE----------------------
#  result2 = dove3(formula = formula,
#                  data = data,
#                  vaccine_infection_interaction = TRUE,
#                  vaccine_uninfected_knots = vaccine.uninfected.knots,
#                  vaccine_infected_knots = vaccine.infected.knots,
#                  prior_infection_knots = prior.infection.knots,
#                  related_vaccine_types = list(c(1,2)))

## ----echo=FALSE---------------------------------------------------------------
load(file = "results2.RData")

## ----idove-constantVE-return-covariates---------------------------------------
result2$covariates

## -----------------------------------------------------------------------------
head(result2$effectiveness[[1]])

## ----eval=FALSE---------------------------------------------------------------
#  result2$plots[[1]]

## ----dove3-interaction-plot, echo=FALSE, out.width='70%',fig.cap='\\label{fig:dove3Figs2}Plots auto-generated by \\textit{dove3()}. The estimated effectiveness curve (black line) under interaction assumption and its $95\\%$ confidence intervals (grey shadow) are shown as a function of the time since vaccination.' , fig.show='hold', fig.align='center'----
knitr::include_graphics(
  path=c("plot2.pdf"),
  auto_pdf = getOption("knitr.graphics.auto_pdf", FALSE),
  dpi = NULL,
  error = getOption("knitr.graphics.error", TRUE)
)

